-----------------------------------------------------------------------------
-- Copyright 2020, Ideas project team. This file is distributed under the
-- terms of the Apache License 2.0. For more information, see the files
-- "LICENSE.txt" and "NOTICE.txt", which are included in the distribution.
-----------------------------------------------------------------------------
module Main where

-- Include the Ideas framework
import Ideas.Common.Library
import Ideas.Main.Default

-- Include the hypothesis exercise and predicate service
import Domain.Hypothesis.Exercises
import Service.ConstraintServices

main :: IO ()
main = defaultMain ideasStatistics

ideasStatistics :: DomainReasoner
ideasStatistics =
  describe "Domain reasoner for statistics" $
  (newDomainReasoner "ideas.statistics")
  { exercises = myExercises
  , services  = myServices
  }

myExercises :: [Some Exercise]
myExercises =
   [ -- hypothesis testing
     Some hypothesisExercise
   ]

myServices :: [Service]
myServices =
  concat [ metaServiceList ideasStatistics
         , filter ((/= newId "basic.diagnose") . getId) serviceList
         , constraintServiceList
         ]